import sys
from PyQt5.QtCore import QSize, Qt, QRect, pyqtSlot
from PyQt5.QtGui import QFont, QPixmap
from PyQt5.QtWidgets import (
    QMainWindow,
    QWidget,
    QSizePolicy,
    QGridLayout,
    QGroupBox,
    QLabel,
    QFrame,
    QLayout,
    QMenuBar,
    QMenu,
    QAction,
    QHBoxLayout,
)

from ezdmb.Utility.ShortcutUtility import setCloseOnEscKey
from ezdmb.View import MenuContentViewUtility


class PreviewWindow(QMainWindow):
    def __init__(self, config, showConfig, showAboutWindow):
        super(self.__class__, self).__init__()
        self.setupUi(showConfig, showAboutWindow)
        self.contentUtil = MenuContentViewUtility.MenuContentViewUtility(
            config,
            self.headerLabel,
            "PreviewWindow",
            self.onRefresh,
        )
        setCloseOnEscKey(self)

    @pyqtSlot(QPixmap)
    def onRefresh(self, value):
        self.headerLabel.setPixmap(value)

    @pyqtSlot(dict)
    def onConfigUpdated(self, data):
        self.contentViewUtil.contentArray = data["imported_content"]
        self.contentViewUtil.rotateContent = data["rotate_content"]
        self.contentViewUtil.rotateTimeout = data["rotate_content_time"]

    def setupUi(self, showConfig, showAboutWindow):
        self.setObjectName("self")
        self.setIconSize(QSize(18, 18))
        self.setDocumentMode(False)

        self.centralWidget = QWidget(self)
        sizePolicy = QSizePolicy(
            QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.centralWidget.sizePolicy().hasHeightForWidth()
        )
        self.centralWidget.setSizePolicy(sizePolicy)
        self.centralWidget.setMinimumSize(QSize(600, 400))
        self.centralWidget.setStyleSheet("")
        self.centralWidget.setObjectName("centralWidget")

        self.gridLayout_2 = QGridLayout(self.centralWidget)
        self.gridLayout_2.setContentsMargins(11, 0, 11, 0)
        self.gridLayout_2.setSpacing(6)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QGridLayout()
        self.gridLayout.setSizeConstraint(QLayout.SetMinAndMaxSize)
        self.gridLayout.setSpacing(6)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_2.addLayout(self.gridLayout, 2, 0, 1, 1)

        self.headerLabel = QLabel(self.centralWidget)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.headerLabel.sizePolicy().hasHeightForWidth()
        )
        self.headerLabel.setSizePolicy(sizePolicy)
        self.headerLabel.setMinimumSize(QSize(400, 200))

        font = QFont()
        font.setFamily("Arial")
        font.setPointSize(24)
        font.setBold(True)
        font.setUnderline(False)
        font.setWeight(75)
        self.headerLabel.setFont(font)
        self.headerLabel.setFrameShape(QFrame.NoFrame)
        self.headerLabel.setAlignment(Qt.AlignCenter)
        self.headerLabel.setText("Digital Menu Board Config")
        self.headerLabel.setObjectName("headerLabel")
        self.gridLayout_2.addWidget(self.headerLabel, 0, 0, 1, 1)

        self.currentMenuGroupBox = QGroupBox(self.centralWidget)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.currentMenuGroupBox.sizePolicy().hasHeightForWidth()
        )
        self.currentMenuGroupBox.setSizePolicy(sizePolicy)
        self.currentMenuGroupBox.setMinimumSize(QSize(400, 400))
        self.currentMenuGroupBox.setTitle("")
        self.currentMenuGroupBox.setFlat(True)
        self.currentMenuGroupBox.setObjectName("currentMenuGroupBox")

        self.horizontalLayout = QHBoxLayout(self.currentMenuGroupBox)
        self.horizontalLayout.setContentsMargins(4, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")

        self.currentMenuLabel = QLabel(self.currentMenuGroupBox)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHeightForWidth(self.currentMenuLabel.sizePolicy().hasHeightForWidth())
        self.currentMenuLabel.setSizePolicy(sizePolicy)
        self.currentMenuLabel.setMinimumSize(QSize(200, 200))
        self.currentMenuLabel.setText("Current Content")
        font = QFont()
        font.setFamily("Segoe UI")
        font.setPointSize(64)
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(50)
        self.currentMenuLabel.setFont(font)
        self.currentMenuLabel.setStyleSheet("font-size: 96px;text-align: center;")
        self.currentMenuLabel.setFrameShape(QFrame.NoFrame)
        self.currentMenuLabel.setFrameShadow(QFrame.Plain)
        self.currentMenuLabel.setLineWidth(0)
        self.currentMenuLabel.setScaledContents(False)
        self.currentMenuLabel.setAlignment(Qt.AlignCenter)
        self.currentMenuLabel.setObjectName("currentMenuLabel")

        self.horizontalLayout.addWidget(self.currentMenuLabel)
        self.gridLayout_2.addWidget(self.currentMenuGroupBox, 1, 0, 1, 1)
        self.setCentralWidget(self.centralWidget)

        self.menuBar = QMenuBar(self)
        self.menuBar.setGeometry(QRect(0, 0, 800, 29))
        self.menuBar.setObjectName("menuBar")
        self.setMenuBar(self.menuBar)

        self.menuFile = QMenu(self.menuBar)
        self.menuFile.setTitle("File")
        self.menuFile.setObjectName("menuFile")

        self.editDisplaySettingsAction = QAction(self)
        self.editDisplaySettingsAction.setText("Edit Display Settings")
        self.editDisplaySettingsAction.setObjectName("editDisplaySettingsAction")
        self.editDisplaySettingsAction.triggered.connect(showConfig)
        self.menuFile.addAction(self.editDisplaySettingsAction)

        self.exitAction = QAction(self)
        self.exitAction.setText("Exit")
        self.exitAction.setObjectName("exitAction")
        self.exitAction.triggered.connect(lambda: sys.exit())
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.exitAction)
        self.menuBar.addAction(self.menuFile.menuAction())

        self.menuHelp = QMenu(self.menuBar)
        self.menuHelp.setTitle("Help")
        self.menuHelp.setObjectName("menuHelp")
        self.showAboutAction = QAction(self)
        self.showAboutAction.setText("About")
        self.showAboutAction.setObjectName("aboutAction")
        self.showAboutAction.triggered.connect(showAboutWindow)
        self.menuHelp.addAction(self.showAboutAction)
        self.menuBar.addAction(self.menuHelp.menuAction())

        self.setWindowTitle("Preview / Configuration")



from PyQt5.QtCore import QSize, Qt
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (
    QCheckBox,
    QDialog,
    QDoubleSpinBox,
    QFileDialog,
    QFrame,
    QGroupBox,
    QHBoxLayout,
    QLabel,
    QListWidget,
    QListWidgetItem,
    QPushButton,
    QSizePolicy,
    QTabWidget,
    QVBoxLayout,
    QWidget
)

from ezdmb.Controller.Configuration import Configuration


class ConfigDialog(QDialog):
    def __init__(self, config: Configuration):
        super(self.__class__, self).__init__()
        self._config = config

        # widgets
        self.setObjectName("ConfigDialog")
        self.setWindowModality(Qt.ApplicationModal)
        self.resize(401, 331)

        preferredSizePolicy = QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding)
        preferredSizePolicy.setHorizontalStretch(0)
        preferredSizePolicy.setVerticalStretch(0)
        preferredSizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(preferredSizePolicy)
        self.setMinimumSize(QSize(400, 300))
        self.vLayout6 = QVBoxLayout(self)
        self.vLayout6.setContentsMargins(0, 0, 0, 0)
        self.vLayout6.setSpacing(0)
        self.vLayout6.setObjectName("vLayout6")

        minMinSizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        minExpMinExpSizePolicy = QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding)
        fixedFixedSizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)

        self.settingsTabs = QTabWidget(self)
        self.settingsTabs.setSizePolicy(minMinSizePolicy)
        self.settingsTabs.setMinimumSize(QSize(200, 180))

        # fonts
        twelvePtFont = QFont()
        twelvePtFont.setPointSize(12)
        twelvePtFont.setBold(False)
        twelvePtFont.setWeight(50)
        fourteenPtFont = QFont()
        fourteenPtFont.setPointSize(14)
        fourteenPtFont.setBold(False)
        fourteenPtFont.setWeight(50)
        sixteenPtFont = QFont()
        sixteenPtFont.setPointSize(16)
        sixteenPtFont.setBold(False)
        sixteenPtFont.setWeight(50)
        twentyPointFont = QFont()
        twentyPointFont.setPointSize(20)

        self.settingsTabs.setFont(twentyPointFont)
        self.settingsTabs.setTabPosition(QTabWidget.North)
        self.settingsTabs.setTabShape(QTabWidget.Rounded)
        self.settingsTabs.setObjectName("settingsTabs")
        self.contentTab = QWidget()
        self.contentTab.setSizePolicy(minExpMinExpSizePolicy)
        self.contentTab.setMinimumSize(QSize(300, 100))
        self.contentTab.setObjectName("contentTab")

        self.vLayout5 = QVBoxLayout(self.contentTab)
        self.vLayout5.setObjectName("vLayout5")
        self.rotationSettingsGrpBox = QGroupBox(self.contentTab)
        preferredSizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        preferredSizePolicy.setHorizontalStretch(0)
        preferredSizePolicy.setVerticalStretch(0)
        preferredSizePolicy.setHeightForWidth(self.rotationSettingsGrpBox.sizePolicy().hasHeightForWidth())
        self.rotationSettingsGrpBox.setSizePolicy(preferredSizePolicy)
        self.rotationSettingsGrpBox.setMinimumSize(QSize(180, 80))
        self.rotationSettingsGrpBox.setFont(sixteenPtFont)
        self.rotationSettingsGrpBox.setObjectName("rotationSettingsGrpBox")

        self.hLayout2 = QHBoxLayout(self.rotationSettingsGrpBox)
        self.hLayout2.setContentsMargins(8, 0, 8, 0)
        self.hLayout2.setSpacing(2)
        self.hLayout2.setObjectName("hLayout2")

        self.rotateImagesCheck = QCheckBox(self.rotationSettingsGrpBox)

        self.rotateImagesCheck.setSizePolicy(fixedFixedSizePolicy)
        self.rotateImagesCheck.setMinimumSize(QSize(180, 24))
        self.rotateImagesCheck.setFont(twelvePtFont)
        self.rotateImagesCheck.setObjectName("rotateImagesCheck")

        self.hLayout2.addWidget(self.rotateImagesCheck)

        self.rotateTimeBox = QDoubleSpinBox(self.rotationSettingsGrpBox)
        self.rotateTimeBox.setSizePolicy(fixedFixedSizePolicy)
        self.rotateTimeBox.setMinimumSize(QSize(70, 24))
        self.rotateTimeBox.setFont(twelvePtFont)
        self.rotateTimeBox.setDecimals(0)
        self.rotateTimeBox.setMinimum(1)
        self.rotateTimeBox.setMaximum(1800)
        self.rotateTimeBox.setObjectName("rotateTimeBox")

        self.hLayout2.addWidget(self.rotateTimeBox)

        self.secondsLabel = QLabel(self.rotationSettingsGrpBox)
        self.secondsLabel.setSizePolicy(minExpMinExpSizePolicy)
        self.secondsLabel.setMinimumSize(QSize(50, 31))
        self.secondsLabel.setFont(twelvePtFont)
        self.secondsLabel.setObjectName("secondsLabel")
        self.hLayout2.addWidget(self.secondsLabel)

        self.vLayout5.addWidget(self.rotationSettingsGrpBox)

        self.addRemoveGrpBox = QGroupBox(self.contentTab)
        self.addRemoveGrpBox.setSizePolicy(minExpMinExpSizePolicy)
        self.addRemoveGrpBox.setMinimumSize(QSize(180, 140))
        self.addRemoveGrpBox.setFont(sixteenPtFont)
        self.addRemoveGrpBox.setObjectName("addRemoveGrpBox")

        self.vLayout2 = QVBoxLayout(self.addRemoveGrpBox)
        self.vLayout2.setContentsMargins(6, 0, 6, 0)
        self.vLayout2.setSpacing(0)
        self.vLayout2.setObjectName("vLayout2")

        self.loadedContentWidget = QListWidget(self.addRemoveGrpBox)
        self.loadedContentWidget.setSizePolicy(minExpMinExpSizePolicy)
        self.loadedContentWidget.setMinimumSize(QSize(200, 20))
        self.loadedContentWidget.setObjectName("loadedContentWidget")
        self.vLayout2.addWidget(self.loadedContentWidget)

        self.frame = QFrame(self.addRemoveGrpBox)
        self.frame.setMinimumSize(QSize(180, 40))
        self.frame.setFrameShape(QFrame.NoFrame)
        self.frame.setFrameShadow(QFrame.Plain)
        self.frame.setObjectName("frame")

        self.horizontalLayout = QHBoxLayout(self.frame)
        self.horizontalLayout.setContentsMargins(6, 3, 6, 8)
        self.horizontalLayout.setSpacing(12)
        self.horizontalLayout.setObjectName("horizontalLayout")

        self.addContentButton = QPushButton(self.frame)
        self.addContentButton.setMinimumSize(QSize(120, 36))
        self.addContentButton.setFont(twelvePtFont)
        self.addContentButton.setObjectName("addContentButton")
        self.horizontalLayout.addWidget(self.addContentButton)

        self.deleteSelectionButton = QPushButton(self.frame)
        self.deleteSelectionButton.setMinimumSize(QSize(120, 36))
        self.deleteSelectionButton.setFont(twelvePtFont)
        self.deleteSelectionButton.setObjectName("deleteSelectionButton")
        self.horizontalLayout.addWidget(self.deleteSelectionButton)

        self.vLayout2.addWidget(self.frame)
        self.vLayout5.addWidget(self.addRemoveGrpBox)

        self.settingsTabs.addTab(self.contentTab, "")
        self.vLayout6.addWidget(self.settingsTabs)

        self.setWindowTitle("Settings")
        self.rotationSettingsGrpBox.setTitle("Rotation settings")
        self.rotateImagesCheck.setText(" Rotate content every")
        self.secondsLabel.setText("seconds")
        self.addRemoveGrpBox.setTitle("Add/Remove content")
        self.addContentButton.setText("Add Content")
        self.deleteSelectionButton.setText("Delete Selection")
        self.settingsTabs.setTabText(self.settingsTabs.indexOf(self.contentTab), "Content")

        self.settingsTabs.setCurrentIndex(0)
        self.setUiFromConfig()

        # signals
        self.addContentButton.clicked.connect(self.addContent)
        self.deleteSelectionButton.clicked.connect(self.deleteSelectedContent)
        self.rotateImagesCheck.stateChanged.connect(self.saveUpdatedConfig)
        self.rotateTimeBox.valueChanged.connect(self.saveUpdatedConfig)

        # Display list of loaded content files for the DMB in the loadedContentWidget
        for i in config.ContentArray:
            item = QListWidgetItem("%s" % str(i))
            self.loadedContentWidget.addItem(item)

    def closeDialog(self):
        self.close()

    def getContentList(self):
        return [
            str(self.loadedContentWidget.item(i).text())
            for i in range(self.loadedContentWidget.count())
        ]

    def saveUpdatedConfig(self):
        self._config.SaveConfig(
            self.rotateImagesCheck.isChecked(),
            float(self.rotateTimeBox.value()),
            self.getContentList(),
        )

    def saveAndClose(self):
        self._config.RotateContent = self.rotateImagesCheck.isChecked()
        self._config.RotateContentTime = self.rotateTimeBox.value()
        self._config.ContentArray = self.getContentList()
        self.saveUpdatedConfig()
        self.close()

    def addContent(self):
        contentFile = QFileDialog.getOpenFileName(self)[0]
        if contentFile != "":
            self.loadedContentWidget.addItem(contentFile)
            self.saveUpdatedConfig()

    def deleteSelectedContent(self):
        for SelectedItem in self.loadedContentWidget.selectedItems():
            self.loadedContentWidget.takeItem(
                self.loadedContentWidget.row(SelectedItem)
            )

        self.saveUpdatedConfig()

    def setUiFromConfig(self):
        self.rotateImagesCheck.setChecked(bool(self._config.RotateContent))
        self.rotateTimeBox.setValue(float(self._config.RotateContentTime))



from os.path import dirname, join, abspath
from setuptools import setup, find_packages

CURDIR = dirname(abspath(__file__))
with open(join(CURDIR, "requirements.txt")) as rs:
    REQS = rs.read().splitlines()

with open(join(CURDIR, "README.md"), "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='ezdmb',
    version="0.9.30",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/justinmichaelvieira/ezdmb",
    project_urls={
        "Bug Tracker": "https://github.com/mahesh-maximus/helloworld-pyp/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3"
    ],
    package_dir={'':"src"},
    packages=find_packages("src"),
    license='LGPL-3.0-only',
    author='Justin Vieira',
    author_email='justin@rancorsoft.com',
    description='A dead-simple digital menu board configurator and display.',
    install_requires=REQS,
    python_requires=">=3.4",
    entry_points={
        "console_scripts": [
            "ezdmb = ezdmb.__main__:main"
        ]
    },
)
